package ru.dbotthepony.mc.otm.core.util

import it.unimi.dsi.fastutil.HashCommon
import net.minecraft.core.component.DataComponentMap
import net.minecraft.world.item.Item
import net.minecraft.world.item.ItemStack

class ItemStackKey(val item: Item, val components: DataComponentMap) {
	// make copy of original itemstack because there is no copy() method on DataComponentMap, which is returned by ItemStack#getComponents
	constructor(itemStack: ItemStack) : this(itemStack.item, itemStack.copy().components)

	private var hashComputed = false
	private var hash = 0

	override fun equals(other: Any?): Boolean {
		return this === other || other is ItemStackKey && other.item === item && other.components == components
	}

	override fun hashCode(): Int {
		if (!hashComputed) {
			hash = HashCommon.murmurHash3(item.hashCode().toLong().shl(32) or components.hashCode().toLong()).toInt()
			hashComputed = true
		}

		return hash
	}

	override fun toString(): String {
		return "ItemStackKey[$item, $components]"
	}
}

fun ItemStack.asKey(): ItemStackKey {
	return ItemStackKey(this)
}
