package ru.dbotthepony.mc.otm.menu.tech

import net.minecraft.world.entity.player.Inventory
import net.minecraft.world.item.ItemStack
import net.neoforged.neoforge.capabilities.Capabilities
import ru.dbotthepony.kommons.util.getValue
import ru.dbotthepony.kommons.util.setValue
import ru.dbotthepony.mc.otm.block.entity.RedstoneSetting
import ru.dbotthepony.mc.otm.block.entity.tech.ChemicalGeneratorBlockEntity
import ru.dbotthepony.mc.otm.container.EnhancedContainer
import ru.dbotthepony.mc.otm.container.slotted.SlottedContainer
import ru.dbotthepony.mc.otm.menu.MatteryMenu
import ru.dbotthepony.mc.otm.menu.MatteryMenuSlot
import ru.dbotthepony.mc.otm.menu.UserFilteredMenuSlot
import ru.dbotthepony.mc.otm.menu.input.EnergyConfigPlayerInput
import ru.dbotthepony.mc.otm.menu.input.EnumInputWithFeedback
import ru.dbotthepony.mc.otm.menu.input.ItemConfigPlayerInput
import ru.dbotthepony.mc.otm.menu.widget.ProfiledLevelGaugeWidget
import ru.dbotthepony.mc.otm.menu.widget.ProgressGaugeWidget
import ru.dbotthepony.mc.otm.registry.game.MMenus

class ChemicalGeneratorMenu @JvmOverloads constructor(id: Int, inv: Inventory, tile: ChemicalGeneratorBlockEntity? = null)
	: MatteryMenu(MMenus.CHEMICAL_GENERATOR, id, inv, tile) {

	val redstoneConfig = EnumInputWithFeedback<RedstoneSetting>(this)
	val itemConfig = ItemConfigPlayerInput(this, tile?.itemConfig, allowPush = true)
	val energyConfig = EnergyConfigPlayerInput(this, tile?.energyConfig, allowPush = true)

	init {
		if (tile != null) {
			redstoneConfig.with(tile.redstoneControl::redstoneSetting)
		}
	}

	val fuelSlot = object : UserFilteredMenuSlot(tile?.fuelContainer ?: SlottedContainer.filtered(1), 0) {
		override fun mayPlace(itemStack: ItemStack): Boolean {
			return itemStack.getBurnTime(null) > 0
		}
	}

	val residueSlot = object : MatteryMenuSlot(tile?.residueContainer ?: EnhancedContainer(1), 0) {
		override fun mayPlace(itemStack: ItemStack): Boolean {
			return false
		}
	}

	val batterySlot = object : UserFilteredMenuSlot(tile?.batteryContainer ?: SlottedContainer.filtered(1), 0) {
		override fun mayPlace(itemStack: ItemStack): Boolean {
			return itemStack.getCapability(Capabilities.EnergyStorage.ITEM)?.canReceive() ?: false
		}
	}

	val progress = ProgressGaugeWidget(this)
	val energy = ProfiledLevelGaugeWidget(this, tile?.energy)
	var burnTime by mSynchronizer.int()

	init {
		addStorageSlot(fuelSlot)
		addStorageSlot(batterySlot)
		addStorageSlot(residueSlot)

		if (tile != null) {
			progress.with { if (tile.workTicksTotal == 0) 0f else 1f - tile.workTicks.toFloat() / tile.workTicksTotal }
		}
	}

	init {
		addInventorySlots()
	}

	override fun broadcastChanges() {
		super.broadcastChanges()
		burnTime = (tile as ChemicalGeneratorBlockEntity).workTicks
	}
}
